unit Utilities;

interface

uses ADODB,Classes,DB,Forms,SysUtils,Variants,Dialogs,CheckLst,Buttons,Graphics,
     fmMainForm;


function IIF( bTrue_False : Boolean; cTrueString , cFalseString : String ) : String; overload;
function IIF( bTrue_False : Boolean; iTrueInteger , iFalseInteger : LongInt ) : LongInt; overload;
function IIF( bTrue_False : Boolean; iTrueReal , iFalseReal : Double ) : Double; overload;
function Replicate(S :string; R:Integer) : String;
function AddSpace(S:String; L:Integer) : String;
function NumI( iValue : LongInt; iLen:Integer):String;

function CreateTableStructure(sTable:String):Boolean;
function GetDataType(iType:Word;iLen:Integer):String;
function GetADOTableSchema(var ADOConn:TADOConnection; const aTable:String):TStrings;
function GetDataTypeString(iDT:TDataType;iLen:Integer):String;
{-----------------------------------------------------------------------------}
function GetItemChecked(chkLst:TCheckListBox):Integer;

implementation

{-----------------------------------------------------------------------------}
function GetItemChecked(chkLst:TCheckListBox):Integer;
var
  i:integer;
begin
  Result:=0;
  for i:=0 to chkLst.Count-1 do
  if chkLst.Checked[i] then Result:=Result+1;
end;
{-----------------------------------------------------------------------------}
function CreateTableStructure(sTable:String):boolean;
  function CreateFromSchema(s:String):Boolean;
  var
    Schema:TStrings;
    surQuery:TADOCommand;
    i:integer;
    bSucc:Boolean;
  begin
    try
      Schema:=TStringList.Create;
      Schema.Assign(GetADOTableSchema(MainForm.ADOConnection1,s));
      surQuery:=TADOCommand.Create(Application);
      if Schema.Count<>0 then
      begin
        with surQuery do
        begin
          Connection:=MainForm.ADOConnection2; // Source Connection
          CommandText:='';
          for i:=0 to Schema.Count-1 do
            CommandText:=CommandText+Schema.Strings[i];
          try
            Execute;
            bSucc:=true;
          except
            bSucc:=false;
          end;
        end;
      end
      else
        bSucc:=false;
    finally
      Result:=bSucc;
      Schema.Free;
      surQuery.Free;
    end;
  end;
  {-----------------------------------------------------------------------------}
  function CreateFromNonSchema(s:String):Boolean;
  var
    surQuery:TADOQuery;
    tarQuery:TADOQuery;
    i:Integer;
    sSQL:String;
    bSucc:Boolean;
  begin
    try
      bSucc:=false;
      surQuery:=TADOQuery.Create(Application);
      tarQuery:=TADOQuery.Create(Application);
      with surQuery do
      begin
        Connection:=MainForm.ADOConnection1;
        MaxRecords:=1;
        Close; SQL.Clear;
        SQL.Add('Select * from '+s);
        Open; sSQL:='Create Table '+s+'(';
        for i:=0 to FieldCount-1 do
        begin
          sSQL:=sSQL+Fields.Fields[i].FieldName+' '+
             GetDataTypeString(Fields.Fields[i].DataType,Fields.Fields[i].Size)+
             ' '+iif(Fields.Fields[i].IsNull,'NULL','NOT NULL');
          if i=FieldCount-1 then
            sSQL:=sSQL+');'
          else
            sSQL:=sSQL+',';
        end;
        with tarQuery do begin
          Connection:=MainForm.ADOConnection2;
          Close; SQL.Clear;
          SQL.Add(sSQL);
          try
            ExecSQL;
            bSucc:=true;
          except
          end;
        end;
      end;
    finally
      Result:=bSucc;
      surQuery.Free;
      tarQuery.Free;
    end;
  end;
var
  bExist , bSucc:Boolean;
begin
  try
    bExist:=MainForm.FTargetFiles.IndexOf(sTable)<>-1;
    if not bExist then
    begin
      bSucc:=CreateFromSchema(sTable);
      if not bSucc then
        bSucc:=CreateFromNonSchema(sTable);
      if bSucc then MainForm.FTargetFiles.Add(sTable);
    end;
  finally
    Result:=bSucc;
  end;
end;
{-----------------------------------------------------------------------------}
function GetDataType(iType:Word;iLen:Integer):String;
var
  s:String;
begin
  case iTYPE of
    $00000000: s := 'Empty';
    $00000010: s := 'TinyInt';
    $00000002: s := 'SMALLINT';
    $00000003: s := 'INTEGER';
    $00000014: s := 'BigInt';
    $00000011: s := 'UnsignedTinyInt';
    $00000012: s := 'UnsignedSmallInt';
    $00000013: s := 'UnsignedInt';
    $00000015: s := 'UnsignedBigInt';
    $00000004: s := 'FLOAT'; //'Single';
    $00000005: s := 'FLOAT'; //'Double';
    $00000006: s := 'Currency';
    $0000000E: s := 'FLOAT'; //'Decimal';
    $00000083: s := 'FLOAT'; //'Numeric';
    $0000000B: s := 'Boolean';
    $0000000A: s := 'Error';
    $00000084: s := 'UserDefined';
    $0000000C: s := 'Variant';
    $00000009: s := 'IDispatch';
    $0000000D: s := 'IUnknown';
    $00000048: s := 'GUID';
    $00000007: s := 'Date';
    $00000085: s := 'DBDate';
    $00000086: s := 'DBTime';
    $00000087: s := 'DATETIME';
    $00000008: s := 'BSTR';
    $00000081: s := iif(iLen<256,'VARCHAR('+Inttostr(iLen)+')','TEXT'); //char
    $000000C8: s := iif(iLen<256,'VARCHAR('+Inttostr(iLen)+')','TEXT'); //varchar
    $000000C9: s := 'LongVarChar';
    $00000082: s := 'WChar';
    $000000CA: s := 'VarWChar';
    $000000CB: s := 'LongVarWChar';
    $00000080: s := 'IMAGE'; //'Binary';
    $000000CC: s := 'IMAGE'; //'VarBinary';
    $000000CD: s := 'LongVarBinary';
    $00000088: s := 'Chapter';
    $00000040: s := 'FileTime';
    $00000089: s := 'DBFileTime';
    $0000008A: s := 'PropVariant';
    $0000008B: s := 'VarNumeric';
  else
    s := '<Unknown>';
  end;
  Result:=s;
end;
{-----------------------------------------------------------------------------}
function GetADOTableSchema(var ADOConn:TADOConnection; const aTable:String):TStrings;
var
  FieldSchema:TADODataSet;
  s,sIndex:String;
begin
  Result:=TStringList.Create;
  try
    Result.Clear; Result.Add('CREATE TABLE '+uppercase(aTable)+'(');
    FieldSchema:=TADODataSet.Create(nil);
    ADOConn.OpenSchema(siPrimaryKeys, VarArrayOf([Unassigned, Unassigned, aTable]), EmptyParam, FieldSchema);

    with FieldSchema do
    begin
      First; sIndex:='';
      while not eof do begin
        sIndex:=sIndex+FieldByName('Column_Name').ASString;
        next;
        if not eof then sIndex:=sIndex+',';
      end;
    end;
    ADOConn.OpenSchema(siColumns, VarArrayOf([Unassigned, Unassigned, aTable, Unassigned]), EmptyParam, FieldSchema);
    with FieldSchema do
    try
      if RecordCount=0 then
        Result.Clear
      else
      begin
        First;
        while not eof do begin
          s:='  '+AddSpace(uppercase(FieldByName('COLUMN_NAME').AsString),20)+' '+
             GetDataType(FieldByName('DATA_TYPE').AsInteger,
                         FieldByName('CHARACTER_MAXIMUM_LENGTH').AsInteger)+
             iif(FieldByName('IS_NULLABLE').AsBoolean,' NULL ',' NOT NULL');
          next;
          if (not eof) or (sIndex<>'')  then
            s:=s+',';
          Result.Add(s);
        end;
        if sIndex<>'' then
          Result.Add(' PRIMARY KEY('+uppercase(sIndex)+')');
          //Result.Add('  CONSTRAINT PK_'+uppercase(aTable)+' PRIMARY KEY('+uppercase(sIndex)+')');
        Result.Add(');');
      end;
    finally
      Free;
    end;
  except
    Result.Clear;
  end;
end;
(*= IIF Block Begin =========================================================*)
function IIF( bTrue_False : Boolean; cTrueString , cFalseString : String ) : String; overload;
begin
    If bTrue_False Then Result:=cTrueString
    Else                Result:=cFalseString;
end;
function IIF( bTrue_False : Boolean; iTrueInteger , iFalseInteger : LongInt ) : LongInt; overload;
begin
    If bTrue_False Then Result:=iTrueInteger
    Else                Result:=iFalseInteger;
end;
function IIF( bTrue_False : Boolean; iTrueReal , iFalseReal : Double ) : Double; overload;
begin
    If bTrue_False Then Result:=iTrueReal
    Else                Result:=iFalseReal;
end;
(*= Block END ==============================================================*)
{=UDF_P=======================================================================}
function Replicate(S :string; R:Integer) : String;
var i : Integer;
    s1: String;
begin
  s1:=''; for i:=1 to R do s1:=s1+S;
  Result:=S1;
end;
{=UDF_P=======================================================================}
function AddSpace(S:String; L:Integer) : String;
begin
  if Length(S)<L then
     S:=S+Replicate(' ',L-Length(S));
  Result:=S;
end;
{=UDF_P=======================================================================}
function NumI( iValue : LongInt; iLen:Integer):String;
var S,S1:String;
    i,j:Integer;
    k : Boolean;
begin
  if iValue<0 then begin  k:=True; iValue:=ABS(iValue); end
  else                    k:=False;

  S1:=IntToStr(iValue); S:=''; j:=0;
  for i:=Length(S1) downto 1 do
    begin
      S:=Copy(S1,i,1)+S;
      INC(j); if (j mod 3=0 ) and (j<>Length(S1)) then S:=','+S;
    end;
  if k then S:='-'+S;
  if Length(S)<iLen then S:=Replicate(' ',iLen-Length(S))+S;
  Result:=S;
end;

{-----------------------------------------------------------------------------}
function GetDataTypeString(iDT:TDataType;iLen:Integer):String;
var
  s:String;
begin
  case iDT of
    ftString:   s:=format('varchar(%d)',[iLen]);
    ftSmallint,
    ftWord:     s:='Smallint';
    ftInteger:  s:='Integer';
    ftFloat:    s:='float';
    ftDate:     s:='Date';
    ftTime:     s:='TIME';
    ftDateTime,
    ftTimeStamp:s:='DATETime';
    ftMemo:     s:='text';
    ftBlob:     s:='Image';
  else
    s := '<Unknown>';
  end;
  Result:=s;
end;

end.
