unit fmMainForm;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,Dialogs,
  ComCtrls, Gauges, StdCtrls, ADODB, ImgList, DB, Buttons, CheckLst,
  ExtCtrls;

type
  TMainForm = class(TForm)
    ADOConnection1: TADOConnection;
    ADOTable1: TADOTable;
    DataSource1: TDataSource;
    ADOConnection2: TADOConnection;
    ADOTable2: TADOTable;
    DataSource2: TDataSource;
    RzPanel1: TPanel;
    Panel2: TPanel;
    RzPanel3: TPanel;
    Panel4: TPanel;
    Panel1: TPanel;
    btnOpenSource: TSpeedButton;
    btnOpenTarget: TSpeedButton;
    btnMover: TSpeedButton;
    ADOQuery1: TADOQuery;
    ImageList1: TImageList;
    btnSelectAll: TSpeedButton;
    btnUnSelect: TSpeedButton;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    Memo1: TMemo;
    btnStop: TSpeedButton;
    btnClose: TSpeedButton;
    ADODataSet1: TADODataSet;
    ADOConnection3: TADOConnection;
    StatusBar: TStatusBar;
    clstSource: TCheckListBox;
    clstTarget: TCheckListBox;
    s1: TImage;
    s2: TImage;
    Panel3: TPanel;
    chkFullTrace: TCheckBox;
    btnClear: TSpeedButton;
    lstMessage: TListBox;
    pbTables: TGauge;
    pbRecords: TProgressBar;
    btnAbout: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure btnOpenSourceClick(Sender: TObject);
    procedure btnOpenTargetClick(Sender: TObject);
    procedure btnMoverClick(Sender: TObject);
    procedure btnUnSelectClick(Sender: TObject);
    procedure btnSelectAllClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure btnStopClick(Sender: TObject);
    procedure StatusBarDrawPanel(StatusBar: TStatusBar; Panel: TStatusPanel;
      const Rect: TRect);
    procedure clstSourceClick(Sender: TObject);
    procedure btnClearClick(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
  private
    { Private declarations }
    bExecuting:Boolean;
    bExecuteStop:Boolean;
    bError:Smallint;
    bitMap:TBitMap;
    function RunSyncDataTable(s:String):boolean;
    procedure AddRecord(s:String);
    procedure ChangeBtnBmp(index:Integer; var Source:TImage);
    procedure DispMessage(sMsg:String);
  public
    { Public declarations }
    FTargetFiles:TStrings; //Target Table List
  end;

const
  aErrorCode:Array[0..3] of String=('','Source Table Open Error!!','Target Table Open Error!!','Append Error!');
  AppTitle='ƶL 0.1';

var
  MainForm: TMainForm;

implementation

uses AdoConEd,Utilities, dgAbout;

{$R *.dfm}
{-----------------------------------------------------------------------------}
procedure TMainForm.DispMessage(sMsg:String);
begin
  lstMessage.ItemIndex:=lstMessage.Items.Add(sMsg);
  Application.ProcessMessages;
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.ChangeBtnBmp(index:Integer; var Source:TImage);
begin
  Source.Picture.Bitmap:=nil;
  ImageList1.GetBitmap(index,Source.Picture.Bitmap);
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.FormCreate(Sender: TObject);
begin
  Caption:='ƶL';
  Application.Title:=AppTitle;
  clstSource.Items.Clear;
  clstTarget.Items.Clear;
  FTargetFiles:=TStringList.Create;
  bitMap:=TBitMap.Create;
  ChangeBtnBmp(0,s1);
  ChangeBtnBmp(0,s2);
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.btnOpenSourceClick(Sender: TObject);
begin
  if bExecuting then
    DispMessage('Moving Data....(Request: Open Source)')
  else
  begin
    with ADOConnection1 do if Connected then Connected:=false;
    ChangeBtnBmp(0,s1);
    EditConnectionString(AdoConnection1);
    try
      DispMessage('Connecting Source Connection');
      ADOConnection1.Connected:=true;
      ADOConnection1.GetTableNames(clstSource.Items);
      btnSelectAll.Enabled:=true;
      btnUnSelect.Enabled:=true;
      ChangeBtnBmp(1,s1);
      DispMessage('Connected Source Connection');
    except
      DispMessage('Connecting Source Connection..(Error:ConnectionString)');
    end;
  end;  
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.btnOpenTargetClick(Sender: TObject);
var
  i:integer;
begin
  if bExecuting then
    DispMessage('Moving Data....(Request: Open Target)')
  else
  begin
    with ADOConnection2 do if Connected then Connected:=false;
    ChangeBtnBmp(0,s2);
    EditConnectionString(AdoConnection2);
    try
      DispMessage('Connecting Target Connection');
      ADOConnection2.Connected:=true;
      ADOConnection2.GetTableNames(FTargetFiles,false);
      for i:=0 to FTargetFiles.Count-1 do
        FTargetFiles.Strings[i]:=lowercase(FTargetFiles.Strings[i]);
      ChangeBtnBmp(1,s2);
      DispMessage('Connected Target Connection');
    except
      DispMessage('Connecting Target Connection..(Error:ConnectionString)');
    end;
  end;
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.btnMoverClick(Sender: TObject);
var
  i: integer;
begin
  if bExecuting then
    DispMessage('Moving Data....(Request: Execute)')
  else
  begin
    if (not ADOConnection1.Connected) or (not ADOConnection2.Connected) then
      MessageDlg('ХOpen Source & Target Connection!', mtError, [mbOK], 0)
    else
    if GetItemChecked(clstSource)=0 then
      MessageDlg('Source table not choice!!', mtError, [mbOK], 0)
    else
    try
      btnStop.Enabled:=true;
      bExecuting:=true;
      clstTarget.Items.Clear;
      Memo1.Lines.Clear;
      pbTables.MaxValue:=GetItemChecked(clstSource);
      pbTables.Progress:=0;
      for i:=0 to clstSource.Items.Count-1 do
        if clstSource.Checked[i] then
        begin
          pbTables.Progress:=pbTables.Progress+1;
          Application.Title:=AppTitle+' T:'+inttostr(pbTables.PercentDone)+'%';
          clstSource.ItemIndex:=i;
          Application.ProcessMessages;
          if not RunSyncDataTable(clstSource.Items.Strings[i]) then
            Break;
        end;
    finally
      btnStop.Enabled:=false;
      bExecuting:=false;
    end;
  end;
end;
{-----------------------------------------------------------------------------}
function TMainForm.RunSyncDataTable(s:String):Boolean;
var
  n:Integer;
begin
  s:=lowercase(s);
  CreateTableStructure(s);
  DispMessage('Preparing Move Table:'+s);
  bError:=0;
  with ADOTable1 do begin
    Close; TableName:=s; try Open; except bError:=1; end;
  end;
  if bError=0 then
    with ADOTable2 do begin
      Close; TableName:=s; try Open; except bError:=2; end;
    end;
  if bError=0 then
    with ADOTable1 do begin
      with ADOQuery1 do begin
        DispMessage('Deleting Target Data');
        Close; SQL.Clear;
        SQL.Add('delete from '+s);
        ExecSQL;
      end;
      DispMessage('Deleted Target Data');
      pbRecords.Max:=RecordCount;
      pbRecords.Position:=1;
      bExecuteStop:=false;
      First;
      DispMessage('Begining Move Source Data to Target');
      while not eof do
      begin
        StatusBar.Panels.Items[4].Text:=format('%d / %d',[pbRecords.Position,pbRecords.Max]);
        pbRecords.Position:=pbRecords.Position+1;
        Application.Title:=AppTitle+' T:'+inttostr(pbTables.PercentDone)+'%' +
                                    ' R:'+
                                    iif(pbRecords.Max=0,'0%',
                                         format('%2.0f%',[pbRecords.Position/pbRecords.Max*100]));
        Application.ProcessMessages;
        try
          AddRecord(s);
        except
        end;
        next;
        if bExecuteStop then
          if MessageDlg('Stop Export?', mtConfirmation, [mbYes,mbNo], 0) = mrYes then
          begin
            Result:=false;
            DispMessage('Move terminated by user !!');
            exit;
          end;
      end;
      DispMessage('End Move Source Data to Target');
   end;
   n:=clstTarget.Items.Add(s+' '+aErrorcode[bError]+' Records: '+Inttostr(pbRecords.Max));
   clstTarget.Checked[n]:=bError=0;
   Result:=true;
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.btnUnSelectClick(Sender: TObject);
var
  i:Integer;
begin
  if bExecuting then
    DispMessage('Moving Data....(Request: UnSelect All Source Tables)')
  else
  begin
    for i:=0 to clstSource.Count-1 do
      clstSource.Checked[i]:=false;
    btnMover.Enabled:=GetItemChecked(clstSource)>0;
  end;
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.btnSelectAllClick(Sender: TObject);
var
  i:Integer;
begin
  if bExecuting then
    DispMessage('Moving Data....(Request: Select All Source Tables)')
  else
  begin
    for i:=0 to clstSource.Count-1 do
      clstSource.Checked[i]:=true;
    btnMover.Enabled:=GetItemChecked(clstSource)>0;
  end;
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.btnCloseClick(Sender: TObject);
begin
  if bExecuting then
    DispMessage('Moving Data....(Request: Exit Program)')
  else
    Close;
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.AddRecord(s:String);
var
  i:integer;
begin
  ADOTable2.Append;
  for i:=0 to ADOTable1.FieldCount-1 do
    try
      ADOTable2.FieldByName(ADOTable1.Fields.Fields[i].FieldName).Value:=ADOTable1.Fields.Fields[i].Value;
    except
      on E: EDatabaseError do
      begin
        DispMessage(E.Message);
        Memo1.Lines.Add('Table: '+s+' '+E.Message);
        bError:=3;
      end;
    end;
  try
    ADOTable2.Post;
  except
    on E: EDatabaseError do
    begin
      DispMessage(E.Message);
      Memo1.Lines.Add('Table: '+s+' '+E.Message);
      bError:=3;
    end;
  end;
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.btnStopClick(Sender: TObject);
begin
  bExecuteStop:=true;
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.StatusBarDrawPanel(StatusBar: TStatusBar;
  Panel: TStatusPanel; const Rect: TRect);
begin
  case Panel.ID of
    1: with pbTables do begin // Tables Progress Component
         Parent:=StatusBar;
         Left:=Rect.Left;
         Top:=Rect.Top;
         Height:=Rect.Bottom-Rect.Top;
         Width:=Rect.Right-Rect.Left;
       end;
    3: with pbRecords do begin // Tables Progress Component
         Parent:=StatusBar;
         Left:=Rect.Left;
         Top:=Rect.Top;
         Height:=Rect.Bottom-Rect.Top;
         Width:=Rect.Right-Rect.Left;
       end;
  end;
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.clstSourceClick(Sender: TObject);
begin
  btnMover.Enabled:=GetItemChecked(clstSource)>0;
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.btnClearClick(Sender: TObject);
begin
  lstMessage.Clear;
end;
{-----------------------------------------------------------------------------}
procedure TMainForm.btnAboutClick(Sender: TObject);
begin
  if bExecuting then
    DispMessage('Moving Data....(Request: Open About)')
  else
    AboutBox.ShowModal;
end;

end.
